#include "p30F6014A.h"
#include "stdio.h"
#include "string.h"
#include "math.h"
#include <time.h>
#include <stdlib.h>

#include <motor_led/e_init_port.h>
#include <motor_led/advance_one_timer/e_led.h>
#include <motor_led/advance_one_timer/e_motors.h>
#include <motor_led/advance_one_timer/e_agenda.h>
#include <uart/e_uart_char.h>
#include <a_d/advance_ad_scan/e_ad_conv.h>
#include <a_d/advance_ad_scan/e_prox.h>
#include <a_d/advance_ad_scan/e_acc.h>
#include <a_d/advance_ad_scan/e_micro.h>
#include <bluetooth/e_bluetooth.h>
#include <motor_led/e_epuck_ports.h>
#include <acc_gyro/e_lsm330.h>
#include <utility/utility.h>
#include "memory.h"
#include <I2C/e_I2C_protocol.h>

/* selector on normal extension*/
#define SELECTOR0 _RG6
#define SELECTOR1 _RG7
#define SELECTOR2 _RG8
#define SELECTOR3 _RG9
extern int selector;

#define CMD_PAUSE_MS 100

int main() {
    
    char buffer[BUFFER_SIZE];
    unsigned char selector = 0;
    unsigned char retValue = 0;
    int i = 0;
    unsigned char masterSentData = 0;
    struct BtEPuck epuck1_robot, epuck2_robot;
    
    epuck2_robot.address[0] = 0x46;
    epuck2_robot.address[1] = 0x52;
    epuck2_robot.address[2] = 0x82;
    epuck2_robot.address[3] = 0xC4;
    epuck2_robot.address[4] = 0x0A;
    epuck2_robot.address[5] = 0x24;
    epuck2_robot.number[0] = '0'; // Always 0000
    epuck2_robot.number[1] = '0';
    epuck2_robot.number[2] = '0';
    epuck2_robot.number[3] = '0';
    epuck2_robot.version = 2;
        
    epuck1_robot.address[0] = 0xFC;
    epuck1_robot.address[1] = 0x4D;
    epuck1_robot.address[2] = 0xD4;
    epuck1_robot.address[3] = 0xE8;
    epuck1_robot.address[4] = 0x00;
    epuck1_robot.address[5] = 0x10;
    epuck1_robot.number[0] = '3'; // Id of the e-puck1
    epuck1_robot.number[1] = '6';
    epuck1_robot.number[2] = '6';
    epuck1_robot.number[3] = '8';
    epuck1_robot.version = 1;    
    
    //system initialization
    e_init_port();    // configure port pins
    e_start_agendas_processing();
    e_init_motors();
    e_init_uart1();   // initialize UART to 115200 Kbaud
    e_init_uart2(BAUD115200);   // initialize UART to 115200 Kbaud
    e_init_ad_scan(ALL_ADC);
    
    //Reset if Power on (some problem for few robots)
    if (RCONbits.POR) {
        RCONbits.POR=0;
        __asm__ volatile ("reset");
    }

    resetTime();

    selector = getselector();

    // If you need to change the PIN of the robot decomment the following code snippet changing the PIN as you want.
    //sprintf(buffer,"%s","3612");
    //e_bt_write_local_pin_number(buffer);
    
    //e_bt_reset_pin(); // Reset to 0000
    
    while (1) {

        switch(selector) {
            case 0: // Master role.
                if(masterSentData == 0) { // Send once.
                    masterSentData = 1;
                    do {
                        e_set_body_led(1);
                        retValue=e_bt_connect_epuck();	// Connect to the first e-puck found. Switch automatically to command mode (piconet master state).
                        e_set_body_led(0);
                        resetTime();
                    } while(retValue!=0);

                    //while(getDiffTimeMs() <= 2000);   // Wait a little to be sure the slave is ready.
                    //resetTime();

                    // The slave is intended to be programmed with the standard firmware and selector placed on position 3; it responds to the "asercom protocol".
                    // Clean the buffer sending a null command.
                    e_set_body_led(1);
                    buffer[0] = '\r';
                    buffer[1] = '\n';
                    e_bt_send_SPP_data(buffer, 2);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    // Request the version to the robot to be sure the buffer is clean.
                    e_set_body_led(1);
                    buffer[0] = 'v';
                    buffer[1] = '\r';
                    buffer[2] = '\n';
                    e_bt_send_SPP_data(buffer, 3);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    e_set_body_led(1);                    
                    memset(buffer, 0x00, BUFFER_SIZE);
                    sprintf(buffer, "L,%x,%x\r", 0, 1); // It seems there is a bug with "%d", use "%x" instead.
                    e_bt_send_SPP_data(buffer, 7);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    for(i=1; i<8; i++) {
                        e_set_body_led(1);                        
                        memset(buffer, 0x00, BUFFER_SIZE);
                        sprintf(buffer, "L,%x,%x\r", i-1, 0); // It seems there is a bug with "%d", use "%x" instead.
                        e_bt_send_SPP_data(buffer, 7);
                        e_set_body_led(0);
                        //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                        //resetTime();
                        while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.
                        
                        e_set_body_led(1);                        
                        memset(buffer, 0x00, BUFFER_SIZE);
                        sprintf(buffer, "L,%x,%x\r", i, 1); // It seems there is a bug with "%d", use "%x" instead.
                        e_bt_send_SPP_data(buffer, 7);
                        e_set_body_led(0);
                        //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                        //resetTime();
                        while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.
                    }

                    e_set_body_led(1);                    
                    memset(buffer, 0x00, BUFFER_SIZE);
                    sprintf(buffer, "L,%x,%x\r", 7, 0); // It seems there is a bug with "%d", use "%x" instead.
                    e_bt_send_SPP_data(buffer, 7);
                    e_set_body_led(0);
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    e_bt_release_SPP_link();

                    e_bt_restore_pin();
                }
                break;

            case 1: // Slave role, just in case you want to use your own protocol. The slave will work in "command mode" (not transparent).
                while(1) {
                            memset(buffer, 0x0, 50);
                            e_bt_recv_SPP_data(buffer);

                            // Handle commands from the master.
                            // ...
                }
                break;

            case 2: // Master role. Connect to e-puck1 without searching.
                if(masterSentData == 0) { // Send once.
                    masterSentData = 1;
                    do {
                        e_set_body_led(1);
                        retValue=e_bt_connect_to_epuck(epuck1_robot.address, epuck1_robot.number, epuck1_robot.version);	// Connect to the first e-puck found. Switch automatically to command mode (piconet master state).
                        e_set_body_led(0);
                        resetTime();
                    } while(retValue!=0);

                    //while(getDiffTimeMs() <= 2000);   // Wait a little to be sure the slave is ready.
                    //resetTime();

                    // The slave is intended to be programmed with the standard firmware and selector placed on position 3; it responds to the "asercom protocol".
                    // Clean the buffer sending a null command.
                    e_set_body_led(1);
                    buffer[0] = '\r';
                    buffer[1] = '\n';
                    e_bt_send_SPP_data(buffer, 2);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    // Request the version to the robot to be sure the buffer is clean.
                    e_set_body_led(1);
                    buffer[0] = 'v';
                    buffer[1] = '\r';
                    buffer[2] = '\n';
                    e_bt_send_SPP_data(buffer, 3);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    e_set_body_led(1);                    
                    memset(buffer, 0x00, BUFFER_SIZE);
                    sprintf(buffer, "L,%x,%x\r", 0, 1); // It seems there is a bug with "%d", use "%x" instead.
                    e_bt_send_SPP_data(buffer, 7);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    for(i=1; i<8; i++) {
                        e_set_body_led(1);                        
                        memset(buffer, 0x00, BUFFER_SIZE);
                        sprintf(buffer, "L,%x,%x\r", i-1, 0); // It seems there is a bug with "%d", use "%x" instead.
                        e_bt_send_SPP_data(buffer, 7);
                        e_set_body_led(0);
                        //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                        //resetTime();
                        while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.
                        
                        e_set_body_led(1);                        
                        memset(buffer, 0x00, BUFFER_SIZE);
                        sprintf(buffer, "L,%x,%x\r", i, 1); // It seems there is a bug with "%d", use "%x" instead.
                        e_bt_send_SPP_data(buffer, 7);
                        e_set_body_led(0);
                        //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                        //resetTime();
                        while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.
                    }

                    e_set_body_led(1);                    
                    memset(buffer, 0x00, BUFFER_SIZE);
                    sprintf(buffer, "L,%x,%x\r", 7, 0); // It seems there is a bug with "%d", use "%x" instead.
                    e_bt_send_SPP_data(buffer, 7);
                    e_set_body_led(0);
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    e_bt_release_SPP_link();

                    e_bt_restore_pin();
                }                
                break;
                
            case 3: // Master role. Connect to e-puck2 without searching.
                if(masterSentData == 0) { // Send once.
                    masterSentData = 1;
                    do {
                        e_set_body_led(1);
                        retValue=e_bt_connect_to_epuck(epuck2_robot.address, epuck2_robot.number, epuck2_robot.version);	// Connect to the first e-puck found. Switch automatically to command mode (piconet master state).
                        e_set_body_led(0);
                        resetTime();
                    } while(retValue!=0);

                    //while(getDiffTimeMs() <= 2000);   // Wait a little to be sure the slave is ready.
                    //resetTime();

                    // The slave is intended to be programmed with the standard firmware and selector placed on position 3; it responds to the "asercom protocol".
                    // Clean the buffer sending a null command.
                    e_set_body_led(1);
                    buffer[0] = '\r';
                    buffer[1] = '\n';
                    e_bt_send_SPP_data(buffer, 2);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    // Request the version to the robot to be sure the buffer is clean.
                    e_set_body_led(1);
                    buffer[0] = 'v';
                    buffer[1] = '\r';
                    buffer[2] = '\n';
                    e_bt_send_SPP_data(buffer, 3);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    e_set_body_led(1);                    
                    memset(buffer, 0x00, BUFFER_SIZE);
                    sprintf(buffer, "L,%x,%x\r", 0, 1); // It seems there is a bug with "%d", use "%x" instead.
                    e_bt_send_SPP_data(buffer, 7);
                    e_set_body_led(0);
                    //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                    //resetTime();
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    for(i=1; i<8; i++) {
                        e_set_body_led(1);                        
                        memset(buffer, 0x00, BUFFER_SIZE);
                        sprintf(buffer, "L,%x,%x\r", i-1, 0); // It seems there is a bug with "%d", use "%x" instead.
                        e_bt_send_SPP_data(buffer, 7);
                        e_set_body_led(0);
                        //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                        //resetTime();
                        while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.
                        
                        e_set_body_led(1);                        
                        memset(buffer, 0x00, BUFFER_SIZE);
                        sprintf(buffer, "L,%x,%x\r", i, 1); // It seems there is a bug with "%d", use "%x" instead.
                        e_bt_send_SPP_data(buffer, 7);
                        e_set_body_led(0);
                        //while(getDiffTimeMs() <= CMD_PAUSE_MS);   // Wait a little before sending the next command.
                        //resetTime();
                        while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.
                    }

                    e_set_body_led(1);                    
                    memset(buffer, 0x00, BUFFER_SIZE);
                    sprintf(buffer, "L,%x,%x\r", 7, 0); // It seems there is a bug with "%d", use "%x" instead.
                    e_bt_send_SPP_data(buffer, 7);
                    e_set_body_led(0);
                    while(!e_bt_recv_SPP_data(buffer)); // Read the data coming from the slave.

                    e_bt_release_SPP_link();

                    e_bt_restore_pin();
                }                
                break;
                
            default:
                break;
                
        }
        
    }

    return 0;

}

